use serde::{Deserialize, Serialize};
use tetratto_shared::{snow::AlmostSnowflake, unix_epoch_timestamp};

#[derive(Serialize, Deserialize)]
pub struct JournalPage {
    pub id: usize,
    pub created: usize,
    pub title: String,
    pub prompt: String,
    /// The ID of the owner of the journal page.
    pub owner: usize,
    /// Who can read the journal page.
    pub read_access: JournalPageReadAccess,
    /// Who can write to the journal page (create journal entries belonging to it).
    ///
    /// The owner of the journal page (and moderators) are the ***only*** people
    /// capable of removing entries.
    pub write_access: JournalPageWriteAccess,
}

impl JournalPage {
    /// Create a new [`JournalPage`].
    pub fn new(title: String, prompt: String, owner: usize) -> Self {
        Self {
            id: AlmostSnowflake::new(1234567890)
                .to_string()
                .parse::<usize>()
                .unwrap(),
            created: unix_epoch_timestamp() as usize,
            title,
            prompt,
            owner,
            read_access: JournalPageReadAccess::default(),
            write_access: JournalPageWriteAccess::default(),
        }
    }
}

/// Who can read a [`JournalPage`].
#[derive(Serialize, Deserialize)]
pub enum JournalPageReadAccess {
    /// Everybody can view the journal page from the owner's profile.
    Everybody,
    /// Only people with the link to the journal page.
    Unlisted,
    /// Only the owner of the journal page.
    Private,
}

impl Default for JournalPageReadAccess {
    fn default() -> Self {
        Self::Everybody
    }
}

/// Who can write to a [`JournalPage`].
#[derive(Serialize, Deserialize)]
pub enum JournalPageWriteAccess {
    /// Everybody (authenticated + anonymous users).
    Everybody,
    /// Authenticated users only.
    Authenticated,
    /// Only the owner of the journal page.
    Owner,
}

impl Default for JournalPageWriteAccess {
    fn default() -> Self {
        Self::Authenticated
    }
}
